#lang racket

(require rackunit)

(define (list-exact-floor l)
  (cond [(empty? l) (list)]
        [else
          ; l = (list 1.1 2.6 3.0)
          (define h (exact-floor (first l))) ; 1
          (define t (list-exact-floor (rest l))) ; (list 2 3)
          (cons h t)
        ]
  ))

; Supplied by the stdlib
(check-equal? 3 (exact-floor 3.14))
(check-equal?
  (list 1 2 3)
  (list-exact-floor (list 1.1 2.6 3.0)))

;(list-exact-floor (list 1.1 2.6 3.0))

(check-equal?
  (list-exact-floor (list 1.1 2.6 3.0))
  (list (exact-floor 1.1) (exact-floor 2.6) (exact-floor 3.0)))


(define (map f l)
  (cond [(empty? l) (list)]
        [else
          ; l = (list 1.1 2.6 3.0)
          (define h (f (first l))) ; 1
          (define t (map f (rest l))) ; (list 2 3)
          (cons h t)
        ]
  ))

(check-equal?
  (map exact-floor (list 1.1 2.6 3.0))
  (list (exact-floor 1.1) (exact-floor 2.6) (exact-floor 3.0)))



#| ; wrong version
(define (reverse l)
  (cond [(empty? l) empty]
        [else
          ; l = '(1 2 3)
          (define h (first l)) ; 1
          (define t (reverse (rest l)))  ; '(3 2)
          (cons t h) ; ( '(3 2) . 1)
          ]
          ))
(reverse '(1 2 3))
|#
(define (reverse l)
  (define (reverse-iter accum l)
    (cond [(empty? l) accum]
          [else
            ; l = (1 2 3)
            ; accum = (list)
            (define h (first l)) ; 1
            (define t (rest l)); '(2 3)
            (reverse-iter (cons h accum) t)]) ; '(3 2 1)
  )
  (reverse-iter (list) l))
(reverse (list 1 2 3))
; (reverse '(1 2 3)
; (reverse-iter '() '(1 2 3))
; (reverse-iter '(1) '(2 3))
; (reverse-iter '(2 1) '(3))
; (reverse-iter '(3 2 1) '())
; '(3 2 1)
(define (map:2 f l)
  (define (map-iter accum l)
    (cond [(empty? l) (reverse accum)]
          [else
            (map-iter (cons (f (first l)) accum) (rest l))]))
  (map-iter (list) l))

;'(define x 1)
'(1.1 2.6 3.0)
(map:2 exact-floor (list 1.1 2.6 3.0))

(define (map:3 f l)
  (define (map-iter accum l)
    (cond [(empty? l) (accum (list))]
          [else
            (map-iter (lambda (rev-l) (accum (cons (f (first l)) rev-l))) (rest l))]))
  (map-iter (lambda (rev-l) rev-l) l))

(map:3 exact-floor (list 1.1 2.6 3.0))

(define (remove-0 l)
  (cond [(empty? l) empty]
        [else
          ; l = (list 1 1 3 0 4)
          (define h (first l)) ; 1
          (define t (remove-0 (rest l))) ; '(1 3 4)
          (cond [(equal? h 0) t] ;
                [else (cons h t)])
        ]))

(check-equal? (list 1 3 4) (remove-0 (list 0 1 3 0 4)))
(check-equal? (list 1 2 3) (remove-0 (list 1 2 3)))

(define (append l1 l2)
  (cond [(empty? l1) l2]
        [else
          (define h (first l1)) ; 0
          (define t (append (rest l1) l2)) ; '(1 2 3 4)
          (cons h t)]))

(check-equal?
  (append (list 0 1 2) (list 3 4))
  (list 0 1 2 3 4))



#|
  (cond [(empty? l) (list)]
        [else
          ; l = (list 1.1 2.6 3.0)
          (define h (f (first l))) ; 1
          (define t (map f (rest l))) ; (list 2 3)
          (cons h t)
        ]
  ))
|#







